@@ -1,7 +1,7 @@
 /*
  * gui-history.c - memorize commands or text for buffers (used by all GUI)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -201,6 +201,9 @@ gui_history_buffer_free (struct t_gui_buffer *buffer)
 {
     struct t_gui_history *ptr_history;
 
+    if (!buffer)
+        return;
+
     while (buffer->history)
     {
         ptr_history = buffer->history->next_history;
@@ -277,11 +280,13 @@ gui_history_hdata_history_update_cb (void *data,
  */
 
 struct t_hdata *
-gui_history_hdata_history_cb (void *data, const char *hdata_name)
+gui_history_hdata_history_cb (const void *pointer, void *data,
+                              const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_history", "next_history",
