@@ -1,7 +1,7 @@
 /*
  * gui-hotlist.c - hotlist management (used by all GUI)
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -59,10 +59,10 @@ int gui_add_hotlist = 1;                    /* 0 is for temporarily disable */
  */
 
 void
-gui_hotlist_changed_signal ()
+gui_hotlist_changed_signal (struct t_gui_buffer *buffer)
 {
     (void) hook_signal_send ("hotlist_changed",
-                             WEECHAT_HOOK_SIGNAL_STRING, NULL);
+                             WEECHAT_HOOK_SIGNAL_POINTER, buffer);
 }
 
 /*
@@ -96,6 +96,11 @@ gui_hotlist_free (struct t_gui_hotlist **hotlist,
 {
     struct t_gui_hotlist *new_hotlist;
 
+    if (!ptr_hotlist)
+        return;
+
+    ptr_hotlist->buffer->hotlist = NULL;
+
     /* remove hotlist from queue */
     if (*last_hotlist == ptr_hotlist)
         *last_hotlist = ptr_hotlist->prev_hotlist;
@@ -190,7 +195,7 @@ gui_hotlist_find_pos (struct t_gui_hotlist *hotlist,
                 if ((new_hotlist->priority > ptr_hotlist->priority)
                     || ((new_hotlist->priority == ptr_hotlist->priority)
                         && (util_timeval_diff (&(new_hotlist->creation_time),
-                                                   &(ptr_hotlist->creation_time)) < 0)))
+                                               &(ptr_hotlist->creation_time)) < 0)))
                     return ptr_hotlist;
             }
             break;
@@ -317,8 +322,7 @@ gui_hotlist_add (struct t_gui_buffer *buffer,
             32,
             WEECHAT_HASHTABLE_STRING,
             WEECHAT_HASHTABLE_POINTER,
-            NULL,
-            NULL);
+            NULL, NULL);
         if (!gui_hotlist_hashtable_add_conditions_pointers)
             return NULL;
     }
@@ -328,8 +332,7 @@ gui_hotlist_add (struct t_gui_buffer *buffer,
             32,
             WEECHAT_HASHTABLE_STRING,
             WEECHAT_HASHTABLE_STRING,
-            NULL,
-            NULL);
+            NULL, NULL);
         if (!gui_hotlist_hashtable_add_conditions_vars)
             return NULL;
     }
@@ -339,8 +342,7 @@ gui_hotlist_add (struct t_gui_buffer *buffer,
             32,
             WEECHAT_HASHTABLE_STRING,
             WEECHAT_HASHTABLE_STRING,
-            NULL,
-            NULL);
+            NULL, NULL);
         if (!gui_hotlist_hashtable_add_conditions_options)
             return NULL;
         hashtable_set (gui_hotlist_hashtable_add_conditions_options,
@@ -380,7 +382,7 @@ gui_hotlist_add (struct t_gui_buffer *buffer,
         if (ptr_hotlist->priority >= priority)
         {
             ptr_hotlist->count[priority]++;
-            gui_hotlist_changed_signal ();
+            gui_hotlist_changed_signal (buffer);
             return ptr_hotlist;
         }
 
@@ -405,6 +407,7 @@ gui_hotlist_add (struct t_gui_buffer *buffer,
     else
         gettimeofday (&(new_hotlist->creation_time), NULL);
     new_hotlist->buffer = buffer;
+    buffer->hotlist = new_hotlist;
     memcpy (new_hotlist->count, count, sizeof (new_hotlist->count));
     new_hotlist->count[priority]++;
     new_hotlist->next_hotlist = NULL;
@@ -412,7 +415,7 @@ gui_hotlist_add (struct t_gui_buffer *buffer,
 
     gui_hotlist_add_hotlist (&gui_hotlist, &last_gui_hotlist, new_hotlist);
 
-    gui_hotlist_changed_signal ();
+    gui_hotlist_changed_signal (buffer);
 
     return new_hotlist;
 }
@@ -468,18 +471,46 @@ gui_hotlist_resort ()
     gui_hotlist = new_hotlist;
     last_gui_hotlist = last_new_hotlist;
 
-    gui_hotlist_changed_signal ();
+    gui_hotlist_changed_signal (NULL);
 }
 
 /*
  * Clears hotlist.
+ *
+ * Argument "level_mask" is a combination of:
+ *   1 = join/part
+ *   2 = message
+ *   4 = private
+ *   8 = highlight
+ *
+ * So for example :
+ *   1 = clear only join/part
+ *   12 = clear only private and highlight
+ *   15 = clear whole hotlist
  */
 
 void
-gui_hotlist_clear ()
+gui_hotlist_clear (int level_mask)
 {
-    gui_hotlist_free_all (&gui_hotlist, &last_gui_hotlist);
-    gui_hotlist_changed_signal ();
+    struct t_gui_hotlist *ptr_hotlist, *ptr_next_hotlist;
+    int hotlist_changed;
+
+    hotlist_changed = 0;
+
+    ptr_hotlist = gui_hotlist;
+    while (ptr_hotlist)
+    {
+        ptr_next_hotlist = ptr_hotlist->next_hotlist;
+        if (level_mask & (1 << ptr_hotlist->priority))
+        {
+            gui_hotlist_free (&gui_hotlist, &last_gui_hotlist, ptr_hotlist);
+            hotlist_changed = 1;
+        }
+        ptr_hotlist = ptr_next_hotlist;
+    }
+
+    if (hotlist_changed)
+        gui_hotlist_changed_signal (NULL);
 }
 
 /*
@@ -531,7 +562,7 @@ gui_hotlist_remove_buffer (struct t_gui_buffer *buffer,
     }
 
     if (hotlist_changed)
-        gui_hotlist_changed_signal ();
+        gui_hotlist_changed_signal (NULL);
 }
 
 /*
@@ -539,11 +570,13 @@ gui_hotlist_remove_buffer (struct t_gui_buffer *buffer,
  */
 
 struct t_hdata *
-gui_hotlist_hdata_hotlist_cb (void *data, const char *hdata_name)
+gui_hotlist_hdata_hotlist_cb (const void *pointer, void *data,
+                              const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_hotlist", "next_hotlist",
