@@ -1,7 +1,7 @@
 /*
  * irc-channel.c - channel and private chat management for IRC plugin
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -33,7 +33,9 @@
 #include "irc-color.h"
 #include "irc-command.h"
 #include "irc-config.h"
+#include "irc-modelist.h"
 #include "irc-nick.h"
+#include "irc-protocol.h"
 #include "irc-server.h"
 #include "irc-input.h"
 
@@ -356,7 +358,7 @@ irc_channel_create_buffer (struct t_irc_server *server,
         }
 
         /* set highlights settings on channel buffer */
-        weechat_buffer_set(
+        weechat_buffer_set (
             ptr_buffer,
             "highlight_words_add",
             (channel_type == IRC_CHANNEL_TYPE_CHANNEL) ?
@@ -426,6 +428,7 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
 {
     struct t_irc_channel *new_channel;
     struct t_gui_buffer *ptr_buffer;
+    const char *ptr_chanmode;
 
     /* create buffer for channel (or use existing one) */
     ptr_buffer = irc_channel_create_buffer (server, channel_type,
@@ -479,6 +482,14 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
     new_channel->nicks_speaking[1] = NULL;
     new_channel->nicks_speaking_time = NULL;
     new_channel->last_nick_speaking_time = NULL;
+    new_channel->modelists = NULL;
+    new_channel->last_modelist = NULL;
+    for (ptr_chanmode = irc_server_get_chanmodes (server); ptr_chanmode[0];
+         ptr_chanmode++)
+    {
+        if (ptr_chanmode[0] != ',')
+            irc_modelist_new (new_channel, ptr_chanmode[0]);
+    }
     new_channel->join_smart_filtered = NULL;
     new_channel->buffer = ptr_buffer;
     new_channel->buffer_as_string = NULL;
@@ -689,8 +700,8 @@ irc_channel_check_whox (struct t_irc_server *server,
 {
     if ((channel->type == IRC_CHANNEL_TYPE_CHANNEL) && channel->nicks)
     {
-        if (server->cap_away_notify
-            || server->cap_account_notify
+        if (weechat_hashtable_has_key (server->cap_list, "away-notify")
+            || weechat_hashtable_has_key (server->cap_list, "account-notify")
             || ((IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK) > 0)
                 && ((IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS) == 0)
                     || (channel->nicks_count <= IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS)))))
@@ -711,8 +722,8 @@ irc_channel_check_whox (struct t_irc_server *server,
         }
         else
         {
-            irc_channel_remove_account(server, channel);
-            irc_channel_remove_away(server, channel);
+            irc_channel_remove_account (server, channel);
+            irc_channel_remove_away (server, channel);
         }
     }
 }
@@ -1084,8 +1095,8 @@ void
 irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
                                         const char *nick)
 {
-    int i, unmask_delay, length_tags, nick_found, join, nick_changed;
-    int smart_filtered, remove_smart_filter;
+    int i, unmask_delay, length_tags, nick_found, join, chghost;
+    int nick_changed, smart_filtered, remove_smart_filter;
     time_t *ptr_time, date_min;
     struct t_hdata *hdata_line, *hdata_line_data;
     struct t_gui_line *own_lines;
@@ -1174,6 +1185,8 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
                 }
                 else if (strcmp (tags[i], "irc_join") == 0)
                     join = 1;
+                else if (strcmp (tags[i], "irc_chghost") == 0)
+                    chghost = 1;
                 else if (strcmp (tags[i], "irc_nick") == 0)
                     nick_changed = 1;
                 else if (strncmp (tags[i], "irc_nick1_", 10) == 0)
@@ -1197,7 +1210,7 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
                     break;
                 remove_smart_filter = 1;
             }
-            else if (nick_found && join && smart_filtered)
+            else if (nick_found && (join || chghost) && smart_filtered)
             {
                 remove_smart_filter = 1;
             }
@@ -1343,7 +1356,11 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
                 weechat_printf_date_tags (
                     ptr_channel->buffer,
                     0,
-                    "irc_nick_back",
+                    irc_protocol_tags (
+                        "nick_back",
+                        NULL,
+                        (nick) ? nick->name : NULL,
+                        (nick) ? nick->host : NULL),
                     _("%s%s%s %s(%s%s%s)%s is back on server"),
                     weechat_prefix ("join"),
                     irc_nick_color_for_msg (server, 1, nick, nickname),
@@ -1359,6 +1376,22 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
     }
 }
 
+/*
+ * Sets state for modelists in a channel.
+ */
+
+void
+irc_channel_modelist_set_state (struct t_irc_channel *channel, int state)
+{
+    struct t_irc_modelist *ptr_modelist;
+
+    for (ptr_modelist = channel->modelists; ptr_modelist;
+         ptr_modelist = ptr_modelist->next_modelist)
+    {
+        ptr_modelist->state = state;
+    }
+}
+
 /*
  * Frees a channel and remove it from channels list.
  */
@@ -1387,6 +1420,7 @@ irc_channel_free (struct t_irc_server *server, struct t_irc_channel *channel)
 
     /* free linked lists */
     irc_nick_free_all (server, channel);
+    irc_modelist_free_all (channel);
 
     /* free channel data */
     if (channel->name)
@@ -1472,6 +1506,8 @@ irc_channel_hdata_channel_cb (const void *pointer, void *data,
         WEECHAT_HDATA_VAR(struct t_irc_channel, nicks_speaking, POINTER, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_channel, nicks_speaking_time, POINTER, 0, NULL, "irc_channel_speaking");
         WEECHAT_HDATA_VAR(struct t_irc_channel, last_nick_speaking_time, POINTER, 0, NULL, "irc_channel_speaking");
+        WEECHAT_HDATA_VAR(struct t_irc_channel, modelists, POINTER, 0, NULL, "irc_modelist");
+        WEECHAT_HDATA_VAR(struct t_irc_channel, last_modelist, POINTER, 0, NULL, "irc_modelist");
         WEECHAT_HDATA_VAR(struct t_irc_channel, join_smart_filtered, HASHTABLE, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_channel, buffer, POINTER, 0, NULL, "buffer");
         WEECHAT_HDATA_VAR(struct t_irc_channel, buffer_as_string, STRING, 0, NULL, NULL);
@@ -1629,6 +1665,7 @@ irc_channel_print_log (struct t_irc_channel *channel)
     struct t_irc_channel_speaking *ptr_nick_speaking;
     int i, index;
     struct t_irc_nick *ptr_nick;
+    struct t_irc_modelist *ptr_modelist;
 
     weechat_log_printf ("");
     weechat_log_printf ("  => channel %s (addr:0x%lx):", channel->name, channel);
@@ -1656,6 +1693,8 @@ irc_channel_print_log (struct t_irc_channel *channel)
     weechat_log_printf ("       nicks_speaking[1]. . . . : 0x%lx", channel->nicks_speaking[1]);
     weechat_log_printf ("       nicks_speaking_time. . . : 0x%lx", channel->nicks_speaking_time);
     weechat_log_printf ("       last_nick_speaking_time. : 0x%lx", channel->last_nick_speaking_time);
+    weechat_log_printf ("       modelists. . . . . . . . : 0x%lx", channel->modelists);
+    weechat_log_printf ("       last_modelist. . . . . . : 0x%lx", channel->last_modelist);
     weechat_log_printf ("       join_smart_filtered. . . : 0x%lx (hashtable: '%s')",
                         channel->join_smart_filtered,
                         weechat_hashtable_get_string (channel->join_smart_filtered,
@@ -1695,4 +1734,9 @@ irc_channel_print_log (struct t_irc_channel *channel)
     {
         irc_nick_print_log (ptr_nick);
     }
+    for (ptr_modelist = channel->modelists; ptr_modelist;
+         ptr_modelist = ptr_modelist->next_modelist)
+    {
+        irc_modelist_print_log (ptr_modelist);
+    }
 }
