@@ -16,6 +16,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * OpenSSL licensing:
+ *
+ *   Additional permission under GNU GPL version 3 section 7:
+ *
+ *   If you modify the Program, or any covered work, by linking or
+ *   combining it with the OpenSSL project's OpenSSL library (or a
+ *   modified version of that library), containing parts covered by the
+ *   terms of the OpenSSL or SSLeay licenses, the licensors of the Program
+ *   grant you additional permission to convey the resulting work.
+ *   Corresponding Source for a non-source form of such a combination
+ *   shall include the source code for the parts of OpenSSL used as well
+ *   as that of the covered work.
  */
 
 #ifndef WEECHAT_PLUGIN_PHP_API_H
@@ -44,6 +57,7 @@ PHP_FUNCTION(weechat_string_match);
 PHP_FUNCTION(weechat_string_has_highlight);
 PHP_FUNCTION(weechat_string_has_highlight_regex);
 PHP_FUNCTION(weechat_string_mask_to_regex);
+PHP_FUNCTION(weechat_string_format_size);
 PHP_FUNCTION(weechat_string_remove_color);
 PHP_FUNCTION(weechat_string_is_command_char);
 PHP_FUNCTION(weechat_string_input_for_buffer);
