@@ -1,7 +1,7 @@
 /*
  * plugin.c - WeeChat plugins management (load/unload dynamic C libraries)
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -423,7 +423,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         gui_chat_printf (NULL,
                          _("%sError: unable to load plugin \"%s\": %s"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
-                         filename, dlerror());
+                         filename, dlerror ());
         gui_chat_printf (NULL,
                          _("%sIf you're trying to load a script and not a C "
                            "plugin, try command to load scripts (/perl, "
@@ -592,6 +592,10 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->initialized = 0;
         ptr_option = config_weechat_debug_get (name);
         new_plugin->debug = (ptr_option) ? CONFIG_INTEGER(ptr_option) : 0;
+        new_plugin->variables = hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING, WEECHAT_HASHTABLE_STRING,
+            NULL, NULL);
 
         /* functions */
         new_plugin->plugin_get_name = &plugin_get_name;
@@ -711,6 +715,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->hashtable_get_string = &hashtable_get_string;
         new_plugin->hashtable_set_pointer = &hashtable_set_pointer;
         new_plugin->hashtable_add_to_infolist = &hashtable_add_to_infolist;
+        new_plugin->hashtable_add_from_infolist = &hashtable_add_from_infolist;
         new_plugin->hashtable_remove = &hashtable_remove;
         new_plugin->hashtable_remove_all = &hashtable_remove_all;
         new_plugin->hashtable_free = &hashtable_free;
@@ -1174,6 +1179,7 @@ plugin_remove (struct t_weechat_plugin *plugin)
         free (plugin->license);
     if (plugin->charset)
         free (plugin->charset);
+    hashtable_free (plugin->variables);
 
     free (plugin);
 
@@ -1406,6 +1412,7 @@ plugin_hdata_plugin_cb (const void *pointer, void *data,
         HDATA_VAR(struct t_weechat_plugin, priority, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_weechat_plugin, initialized, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_weechat_plugin, debug, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_weechat_plugin, variables, HASHTABLE, 0, NULL, NULL);
         HDATA_VAR(struct t_weechat_plugin, prev_plugin, POINTER, 0, NULL, hdata_name);
         HDATA_VAR(struct t_weechat_plugin, next_plugin, POINTER, 0, NULL, hdata_name);
         HDATA_LIST(weechat_plugins, WEECHAT_HDATA_LIST_CHECK_POINTERS);
@@ -1463,6 +1470,8 @@ plugin_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!infolist_new_var_integer (ptr_item, "debug", plugin->debug))
         return 0;
+    if (!hashtable_add_to_infolist (plugin->variables, ptr_item, "var"))
+        return 0;
 
     return 1;
 }
@@ -1485,11 +1494,14 @@ plugin_print_log ()
         log_printf ("  handle . . . . . . . . : 0x%lx", ptr_plugin->handle);
         log_printf ("  name . . . . . . . . . : '%s'",  ptr_plugin->name);
         log_printf ("  description. . . . . . : '%s'",  ptr_plugin->description);
+        log_printf ("  author . . . . . . . . : '%s'",  ptr_plugin->author);
         log_printf ("  version. . . . . . . . : '%s'",  ptr_plugin->version);
+        log_printf ("  license. . . . . . . . : '%s'",  ptr_plugin->license);
         log_printf ("  charset. . . . . . . . : '%s'",  ptr_plugin->charset);
         log_printf ("  priority . . . . . . . : %d",    ptr_plugin->priority);
         log_printf ("  initialized. . . . . . : %d",    ptr_plugin->initialized);
         log_printf ("  debug. . . . . . . . . : %d",    ptr_plugin->debug);
+        hashtable_print_log (ptr_plugin->variables, "variables");
         log_printf ("  prev_plugin. . . . . . : 0x%lx", ptr_plugin->prev_plugin);
         log_printf ("  next_plugin. . . . . . : 0x%lx", ptr_plugin->next_plugin);
     }
