@@ -1,7 +1,7 @@
 /*
  * relay-irc.c - IRC protocol for relay to client: IRC proxy/bouncer
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -599,7 +599,7 @@ relay_irc_get_line_info (struct t_relay_client *client,
                          char **tags, char **message)
 {
     int i, num_tags, command, action, all_tags, length;
-    char str_tag[256], *pos, *message_no_color, str_time[256];
+    char str_tag[512], *pos, *message_no_color, str_time[256];
     const char *ptr_tag, *ptr_message, *ptr_nick, *ptr_nick1, *ptr_nick2;
     const char *ptr_host, *localvar_nick, *time_format;
     time_t msg_date;
@@ -1364,7 +1364,7 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
     /* server capabilities */
     if (irc_command && (weechat_strcasecmp (irc_command, "cap") == 0))
     {
-        if ((irc_argc > 0) && irc_argv)
+        if (irc_argc > 0)
         {
             relay_irc_recv_command_capab (client,
                                           irc_argc, irc_argv, irc_argv_eol);
@@ -1499,7 +1499,7 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
                              "weechat-relay-irc, running version %s",
                              RELAY_IRC_DATA(client, address),
                              RELAY_IRC_DATA(client, nick),
-                             weechat_info_get("version", NULL));
+                             weechat_info_get ("version", NULL));
             snprintf (str_time, sizeof (str_time), "%s",
                       ctime (&client->listen_start_time));
             if (str_time[0])
@@ -1514,7 +1514,7 @@ relay_irc_recv (struct t_relay_client *client, const char *data)
                              RELAY_IRC_DATA(client, address),
                              RELAY_IRC_DATA(client, nick),
                              RELAY_IRC_DATA(client, address),
-                             weechat_info_get("version", NULL));
+                             weechat_info_get ("version", NULL));
             infolist_server = weechat_infolist_get ("irc_server", NULL,
                                                     client->protocol_args);
             if (infolist_server)
