@@ -1,7 +1,7 @@
 /*
  * test-string.cpp - test string functions
  *
- * Copyright (C) 2014-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2018 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -164,6 +164,134 @@ TEST(String, Case)
     free (str);
 }
 
+/*
+ * Tests functions:
+ *   string_cut
+ */
+
+TEST(String, Cut)
+{
+    POINTERS_EQUAL(NULL, string_cut (NULL, 0, 0, 0, NULL));
+    STRCMP_EQUAL("", string_cut ("", 0, 0, 0, NULL));
+
+    /* cut with length == 0 */
+    STRCMP_EQUAL("", string_cut ("noël", 0, 0, 0, NULL));
+    STRCMP_EQUAL("+", string_cut ("noël", 0, 0, 0, "+"));
+    STRCMP_EQUAL("…", string_cut ("noël", 0, 0, 0, "…"));
+    STRCMP_EQUAL("", string_cut ("noël", 0, 1, 0, NULL));
+    STRCMP_EQUAL("", string_cut ("noël", 0, 1, 0, "+"));
+    STRCMP_EQUAL("", string_cut ("noël", 0, 1, 0, "…"));
+
+    /* cut with length == 1 */
+    STRCMP_EQUAL("n", string_cut ("noël", 1, 0, 0, NULL));
+    STRCMP_EQUAL("n+", string_cut ("noël", 1, 0, 0, "+"));
+    STRCMP_EQUAL("n…", string_cut ("noël", 1, 0, 0, "…"));
+    STRCMP_EQUAL("n", string_cut ("noël", 1, 1, 0, NULL));
+    STRCMP_EQUAL("+", string_cut ("noël", 1, 1, 0, "+"));
+    STRCMP_EQUAL("…", string_cut ("noël", 1, 1, 0, "…"));
+
+    /* cut with length == 2 */
+    STRCMP_EQUAL("no", string_cut ("noël", 2, 0, 0, NULL));
+    STRCMP_EQUAL("no+", string_cut ("noël", 2, 0, 0, "+"));
+    STRCMP_EQUAL("no…", string_cut ("noël", 2, 0, 0, "…"));
+    STRCMP_EQUAL("no", string_cut ("noël", 2, 1, 0, NULL));
+    STRCMP_EQUAL("n+", string_cut ("noël", 2, 1, 0, "+"));
+    STRCMP_EQUAL("n…", string_cut ("noël", 2, 1, 0, "…"));
+
+    /* cut with length == 3 */
+    STRCMP_EQUAL("noë", string_cut ("noël", 3, 0, 0, NULL));
+    STRCMP_EQUAL("noë+", string_cut ("noël", 3, 0, 0, "+"));
+    STRCMP_EQUAL("noë…", string_cut ("noël", 3, 0, 0, "…"));
+    STRCMP_EQUAL("noë", string_cut ("noël", 3, 1, 0, NULL));
+    STRCMP_EQUAL("no+", string_cut ("noël", 3, 1, 0, "+"));
+    STRCMP_EQUAL("no…", string_cut ("noël", 3, 1, 0, "…"));
+
+    /* cut with length == 4 */
+    STRCMP_EQUAL("noël", string_cut ("noël", 4, 0, 0, NULL));
+    STRCMP_EQUAL("noël", string_cut ("noël", 4, 0, 0, "+"));
+    STRCMP_EQUAL("noël", string_cut ("noël", 4, 0, 0, "…"));
+    STRCMP_EQUAL("noël", string_cut ("noël", 4, 1, 0, NULL));
+    STRCMP_EQUAL("noël", string_cut ("noël", 4, 1, 0, "+"));
+    STRCMP_EQUAL("noël", string_cut ("noël", 4, 1, 0, "…"));
+
+    /* cut with length == 5 */
+    STRCMP_EQUAL("noël", string_cut ("noël", 5, 0, 0, NULL));
+    STRCMP_EQUAL("noël", string_cut ("noël", 5, 0, 0, "+"));
+    STRCMP_EQUAL("noël", string_cut ("noël", 5, 0, 0, "…"));
+    STRCMP_EQUAL("noël", string_cut ("noël", 5, 1, 0, NULL));
+    STRCMP_EQUAL("noël", string_cut ("noël", 5, 1, 0, "+"));
+    STRCMP_EQUAL("noël", string_cut ("noël", 5, 1, 0, "…"));
+
+    /* cut with length == 1, screen == 0 then 1 */
+    STRCMP_EQUAL("こ", string_cut ("こんにちは世界", 1, 0, 0, NULL));
+    STRCMP_EQUAL("こ+", string_cut ("こんにちは世界", 1, 0, 0, "+"));
+    STRCMP_EQUAL("こ…", string_cut ("こんにちは世界", 1, 0, 0, "…"));
+    STRCMP_EQUAL("こ", string_cut ("こんにちは世界", 1, 1, 0, NULL));
+    STRCMP_EQUAL("+", string_cut ("こんにちは世界", 1, 1, 0, "+"));
+    STRCMP_EQUAL("…", string_cut ("こんにちは世界", 1, 1, 0, "…"));
+
+    STRCMP_EQUAL("", string_cut ("こんにちは世界", 1, 0, 1, NULL));
+    STRCMP_EQUAL("+", string_cut ("こんにちは世界", 1, 0, 1, "+"));
+    STRCMP_EQUAL("…", string_cut ("こんにちは世界", 1, 0, 1, "…"));
+    STRCMP_EQUAL("", string_cut ("こんにちは世界", 1, 1, 1, NULL));
+    STRCMP_EQUAL("+", string_cut ("こんにちは世界", 1, 1, 1, "+"));
+    STRCMP_EQUAL("…", string_cut ("こんにちは世界", 1, 1, 1, "…"));
+
+    /* cut with length == 2, screen == 0 then 1 */
+    STRCMP_EQUAL("こん", string_cut ("こんにちは世界", 2, 0, 0, NULL));
+    STRCMP_EQUAL("こん+", string_cut ("こんにちは世界", 2, 0, 0, "+"));
+    STRCMP_EQUAL("こん…", string_cut ("こんにちは世界", 2, 0, 0, "…"));
+    STRCMP_EQUAL("こん", string_cut ("こんにちは世界", 2, 1, 0, NULL));
+    STRCMP_EQUAL("こ+", string_cut ("こんにちは世界", 2, 1, 0, "+"));
+    STRCMP_EQUAL("こ…", string_cut ("こんにちは世界", 2, 1, 0, "…"));
+
+    STRCMP_EQUAL("こ", string_cut ("こんにちは世界", 2, 0, 1, NULL));
+    STRCMP_EQUAL("こ+", string_cut ("こんにちは世界", 2, 0, 1, "+"));
+    STRCMP_EQUAL("こ…", string_cut ("こんにちは世界", 2, 0, 1, "…"));
+    STRCMP_EQUAL("こ", string_cut ("こんにちは世界", 2, 1, 1, NULL));
+    STRCMP_EQUAL("+", string_cut ("こんにちは世界", 2, 1, 1, "+"));
+    STRCMP_EQUAL("…", string_cut ("こんにちは世界", 2, 1, 1, "…"));
+
+    /* cut with length == 3, screen == 0 then 1 */
+    STRCMP_EQUAL("こんに", string_cut ("こんにちは世界", 3, 0, 0, NULL));
+    STRCMP_EQUAL("こんに+", string_cut ("こんにちは世界", 3, 0, 0, "+"));
+    STRCMP_EQUAL("こんに…", string_cut ("こんにちは世界", 3, 0, 0, "…"));
+    STRCMP_EQUAL("こんに", string_cut ("こんにちは世界", 3, 1, 0, NULL));
+    STRCMP_EQUAL("こん+", string_cut ("こんにちは世界", 3, 1, 0, "+"));
+    STRCMP_EQUAL("こん…", string_cut ("こんにちは世界", 3, 1, 0, "…"));
+
+    STRCMP_EQUAL("こ", string_cut ("こんにちは世界", 3, 0, 1, NULL));
+    STRCMP_EQUAL("こ+", string_cut ("こんにちは世界", 3, 0, 1, "+"));
+    STRCMP_EQUAL("こ…", string_cut ("こんにちは世界", 3, 0, 1, "…"));
+    STRCMP_EQUAL("こ", string_cut ("こんにちは世界", 3, 1, 1, NULL));
+    STRCMP_EQUAL("こ+", string_cut ("こんにちは世界", 3, 1, 1, "+"));
+    STRCMP_EQUAL("こ…", string_cut ("こんにちは世界", 3, 1, 1, "…"));
+}
+
+/*
+ * Tests functions:
+ *   string_reverse
+ */
+
+TEST(String, Reverse)
+{
+    POINTERS_EQUAL(NULL, string_reverse (NULL));
+    STRCMP_EQUAL("", string_reverse (""));
+
+    /* reverse of UTF-8 string */
+    STRCMP_EQUAL("n", string_reverse ("n"));
+    STRCMP_EQUAL("on", string_reverse ("no"));
+    STRCMP_EQUAL("ëon", string_reverse ("noë"));
+    STRCMP_EQUAL("lëon", string_reverse ("noël"));
+    STRCMP_EQUAL("界世はちにんこ", string_reverse ("こんにちは世界"));
+
+    /*
+     * reverse of ISO-8859-15 string: the result may not be what you expect:
+     * the function string_reverse accepts only an UTF-8 string as input
+     */
+    STRCMP_EQUAL("\xeblon", string_reverse ("no\xebl"));
+}
+
 /*
  * Tests functions:
  *   string_strcasecmp
@@ -575,29 +703,29 @@ TEST(String, Regex)
     string_regex_flags ("", REG_EXTENDED, &flags);
     LONGS_EQUAL(REG_EXTENDED, flags);
 
-    ptr = string_regex_flags ("test", REG_EXTENDED, &flags);
+    ptr = string_regex_flags ("test1", REG_EXTENDED, &flags);
     LONGS_EQUAL(REG_EXTENDED, flags);
-    STRCMP_EQUAL("test", ptr);
+    STRCMP_EQUAL("test1", ptr);
 
-    string_regex_flags ("(?e)test", 0, &flags);
+    ptr = string_regex_flags ("(?e)test2", 0, &flags);
     LONGS_EQUAL(REG_EXTENDED, flags);
-    STRCMP_EQUAL("test", ptr);
+    STRCMP_EQUAL("test2", ptr);
 
-    string_regex_flags ("(?ei)test", 0, &flags);
+    ptr = string_regex_flags ("(?ei)test3", 0, &flags);
     LONGS_EQUAL(REG_EXTENDED | REG_ICASE, flags);
-    STRCMP_EQUAL("test", ptr);
+    STRCMP_EQUAL("test3", ptr);
 
-    string_regex_flags ("(?eins)test", 0, &flags);
+    ptr = string_regex_flags ("(?eins)test4", 0, &flags);
     LONGS_EQUAL(REG_EXTENDED | REG_ICASE | REG_NEWLINE | REG_NOSUB, flags);
-    STRCMP_EQUAL("test", ptr);
+    STRCMP_EQUAL("test4", ptr);
 
-    string_regex_flags ("(?ins)test", REG_EXTENDED, &flags);
+    ptr = string_regex_flags ("(?ins)test5", REG_EXTENDED, &flags);
     LONGS_EQUAL(REG_EXTENDED | REG_ICASE | REG_NEWLINE | REG_NOSUB, flags);
-    STRCMP_EQUAL("test", ptr);
+    STRCMP_EQUAL("test5", ptr);
 
-    string_regex_flags ("(?ins-e)test", REG_EXTENDED, &flags);
+    ptr = string_regex_flags ("(?ins-e)test6", REG_EXTENDED, &flags);
     LONGS_EQUAL(REG_ICASE | REG_NEWLINE | REG_NOSUB, flags);
-    STRCMP_EQUAL("test", ptr);
+    STRCMP_EQUAL("test6", ptr);
 
     /* compile regular expression */
     LONGS_EQUAL(-1, string_regcomp (&regex, NULL, 0));
